@@ -21,19 +21,21 @@
  */
 
 #include <math.h>
+#include <time.h>
+#include <pds/pdsfuncs.h>
 #include <pds/pdsninput.h>
 #include <pds/pdsneuron.h>
 
 
-////////////////////////////////////////////////////////////////////////////////
-////  Trabajando con PdsNeuron                                              ////
-////////////////////////////////////////////////////////////////////////////////
-
 
 /** \fn PdsNeuron *pds_neuron_new(PdsSnNatural Nel)
- *  \brief Crea un vector de tipo PdsNeuron e inicia con cero todos los elementos
- *  excepto Sigma \$ = \sigma \$ que toma valor uno.
- *  \param[in] Nel Es el número de elementos del vector.
+ *  \brief Crea una neurona de tipo PdsNeuron de Nel entradas.
+ *
+ *  Los valores por defecto son :
+ *  \f[ U=1.0 \f]
+ *  \f[ \sigma=1.0 \f]
+ *  \f[ func(x)=tanh\left(\frac{x}{2}\right) \f]
+ *  \param[in] Nel Es el número de entradas de la neurona.
  *  \return Un puntero al vector de tipo PdsNeuron.
  *  \ingroup PdsNeuronGroup
  */
@@ -65,9 +67,65 @@ PdsNeuron *pds_neuron_new(PdsSnNatural Nel)
 
 	for(i=0;i<Neuron->Nel;i++)	Neuron->W[i]=0.0;
 	
-	Neuron->U=0.0;
+	Neuron->U=1.0;
 	Neuron->Sigma=1.0;
 	Neuron->Theta=0.0;
+    Neuron->func=pds_func;
+    Neuron->dfunc=pds_dfunc;
+
+	Neuron->F=0.0;
+	Neuron->Y[0]=0.0;
+	Neuron->Y[1]=0.0;
+
+	return Neuron;
+}
+
+/** \fn PdsNeuron *pds_neuron_new_with_params(PdsSnNatural Nel,PdsSnReal W,PdsSnReal U,PdsSnReal Sigma,double (*func)(double),double (*dfunc)(double));
+ *  \brief Crea una neurona de tipo PdsNeuron de Nel entradas.
+ *
+ *  \param[in] Nel Es el número de entradas de la neurona.
+ *  \param[in] W Peso inicial de todas las entradas
+ *  \param[in] U Umbral de disparo.
+ *  \param[in] Sigma Factor de escala de la función de activación.
+ *  \param[in] func Función de activación.
+ *  \param[in] dfunc Derivada de la función de activación.
+ *  \return Un puntero al vector de tipo PdsNeuron.
+ *  \ingroup PdsNeuronGroup
+ */
+PdsNeuron *pds_neuron_new_with_params(PdsSnNatural Nel,PdsSnReal W,PdsSnReal U,PdsSnReal Sigma,double (*func)(double),double (*dfunc)(double))
+{
+	PdsNeuron *Neuron=NULL;
+	PdsSnNatural i;
+
+	if(Nel<=0)		return NULL;
+	
+	Neuron=(PdsNeuron *)calloc(1,sizeof(PdsNeuron));
+	if(Neuron==NULL)	return NULL;
+
+	Neuron->Nel=Nel;
+	Neuron->X=(PdsSnReal **)calloc(Neuron->Nel,sizeof(PdsSnReal*));
+	if(Neuron->X==NULL) 
+	{
+		free(Neuron);
+		return NULL;
+	}
+	for(i=0;i<Neuron->Nel;i++)	Neuron->X[i]=NULL;
+
+	Neuron->W=(PdsSnReal *)calloc(Neuron->Nel,sizeof(PdsSnReal));
+	if(Neuron->W==NULL) 
+	{
+		free(Neuron);
+		return NULL;
+	}
+
+	for(i=0;i<Neuron->Nel;i++)	Neuron->W[i]=W;
+	
+	Neuron->U=U;
+	Neuron->Sigma=Sigma;
+	Neuron->Theta=0.0;
+    Neuron->func=func;
+    Neuron->dfunc=dfunc;
+
 	Neuron->F=0.0;
 	Neuron->Y[0]=0.0;
 	Neuron->Y[1]=0.0;
@@ -76,7 +134,7 @@ PdsNeuron *pds_neuron_new(PdsSnNatural Nel)
 }
 
 
-/** \fn int pds_neuron_get_synaptic_weight(const PdsNeuron *Neuron, PdsSnNatural x, PdsSnReal *m)
+/** \fn int pds_neuron_get_weight(const PdsNeuron *Neuron, PdsSnNatural x, PdsSnReal *m)
  *  \brief Devuelve el valor en la posición (x) del vector de pesos de la Neurona.
  *  (x) inicia con (0).
  *  \param[in] Neuron La neurona en consulta.
@@ -86,7 +144,7 @@ PdsNeuron *pds_neuron_new(PdsSnNatural Nel)
  *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
  *  \ingroup PdsNeuronGroup
  */
-int pds_neuron_get_synaptic_weight(const PdsNeuron *Neuron, PdsSnNatural x, PdsSnReal *m)
+int pds_neuron_get_weight(const PdsNeuron *Neuron, PdsSnNatural x, PdsSnReal *m)
 {
 	*m=0;
 
@@ -99,7 +157,7 @@ int pds_neuron_get_synaptic_weight(const PdsNeuron *Neuron, PdsSnNatural x, PdsS
 }
 
 
-/** \fn int pds_neuron_set_synaptic_weight(PdsNeuron *Neuron, PdsSnNatural x, PdsSnReal m)
+/** \fn int pds_neuron_set_weight(PdsNeuron *Neuron, PdsSnNatural x, PdsSnReal m)
  *  \brief Escribe el valor m en la posición (x) del vector de pesos de la Neurona.
  *  (x) inicia con (0).
  *  \param[in,out] Neuron La neurona a escribir.
@@ -109,7 +167,7 @@ int pds_neuron_get_synaptic_weight(const PdsNeuron *Neuron, PdsSnNatural x, PdsS
  *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
  *  \ingroup PdsNeuronGroup
  */
-int pds_neuron_set_synaptic_weight(PdsNeuron *Neuron, PdsSnNatural x, PdsSnReal m)
+int pds_neuron_set_weight(PdsNeuron *Neuron, PdsSnNatural x, PdsSnReal m)
 {
 	if(Neuron==NULL)	return FALSE;
 
@@ -119,14 +177,14 @@ int pds_neuron_set_synaptic_weight(PdsNeuron *Neuron, PdsSnNatural x, PdsSnReal
 }
 
 
-/** \fn int pds_neuron_init_synaptic_weight(PdsNeuron *Neuron, PdsSnReal m)
+/** \fn int pds_neuron_init_weight(PdsNeuron *Neuron, PdsSnReal m)
  *  \brief Inicia el vector de pesos de la Neurona.
  *  \param[in,out] Neuron La neurona a escribir.
  *  \param[in] m El valor de los pesos de la neurona.
  *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
  *  \ingroup PdsNeuronGroup
  */
-int pds_neuron_init_synaptic_weight(PdsNeuron *Neuron, PdsSnReal m)
+int pds_neuron_init_weight(PdsNeuron *Neuron, PdsSnReal m)
 {
 	PdsSnNatural i;
 
@@ -138,6 +196,33 @@ int pds_neuron_init_synaptic_weight(PdsNeuron *Neuron, PdsSnReal m)
 }
 
 
+/** \fn int pds_neuron_init_weight_rand(PdsNeuron *Neuron, PdsSnReal min, PdsSnReal max)
+ *  \brief Inicia el vector de pesos de la Neurona con valores aleatorios
+ *  uniformemente distribuido entre min y max.
+ *  \param[in,out] Neuron La neurona a escribir.
+ *  \param[in] min El minimo valor encontrado en los pesos de la neurona.
+ *  \param[in] max El minimo valor encontrado en los pesos de la neurona.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
+ *  \ingroup PdsNeuronGroup
+ */
+int pds_neuron_init_weight_rand(PdsNeuron *Neuron, PdsSnReal min, PdsSnReal max)
+{
+	PdsSnNatural i;
+    PdsSnReal m;
+
+	if(Neuron==NULL)	return FALSE;
+
+    srand(clock());
+	for(i=0;i<Neuron->Nel;i++)
+    {
+        m = min+(rand()*(max-min))/RAND_MAX;
+	    Neuron->W[i]=m;
+    }
+
+	return TRUE;
+}
+
+
 /** \fn int pds_neuron_get_number_of_inputs(const PdsNeuron *Neuron,PdsSnNatural *Nelements)
  *  \brief Devuelve el número de entradas de la Neurona.
  *  \param[in] Neuron La neurona en consulta.
@@ -232,14 +317,14 @@ int pds_neuron_set_u(PdsNeuron *Neuron,PdsSnReal U)
 }
 
 
-/** \fn PdsSnReal *pds_neuron_get_dendrite(const PdsNeuron *Neuron,PdsSnNatural id)
+/** \fn const PdsSnReal *pds_neuron_get_dendrite(const PdsNeuron *Neuron,PdsSnNatural id)
  *  \brief Devuelve la dirección de la entrada X[id] de la Neurona.
  *  \param[in] Neuron La neurona en consulta.
  *  \param[in] id La posición a consultar, id tiene que ser menor que Nel.
  *  \return La dirección de la entrada X[id] o NULL en caso de error (ej: Neuron==NULL). 
  *  \ingroup PdsNeuronGroup
  */
-PdsSnReal *pds_neuron_get_dendrite(const PdsNeuron *Neuron,PdsSnNatural id)
+const PdsSnReal *pds_neuron_get_dendrite(const PdsNeuron *Neuron,PdsSnNatural id)
 {
 	if(Neuron==NULL)		return NULL;
 	if((id>=Neuron->Nel)||(id<0))	return NULL;
@@ -300,6 +385,28 @@ int pds_neuron_get_output(const PdsNeuron *Neuron,PdsSnReal *m)
 }
 
 
+
+/** \fn int pds_neuron_connect_input_with_ninput(PdsNeuron *Neuron,PdsSnNatural id,PdsNInput *NInput)
+ *  \brief Conecta el valor de salida Y de la neurona de entrada NInput con la 
+ *  entrada X[id] de la neurona Neuron.
+ *  \param[in,out] Neuron La neurona a trabajar.
+ *  \param[in] id La posición a conectar, id tiene que ser menor que Nel.
+ *  \param[in] NInput La neurona de entrada que se conectará a la entrada X[id] de Neuron.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
+ *  \ingroup PdsNeuronGroup
+ */
+int pds_neuron_connect_input_with_ninput(PdsNeuron *Neuron,PdsSnNatural id,PdsNInput *NInput)
+{
+	if(Neuron==NULL)		return FALSE;
+	if(NInput==NULL)		return FALSE;
+	if((id>=Neuron->Nel)||(id<0))	return FALSE;
+
+	Neuron->X[id]=NInput->Y;
+
+	return TRUE;
+}
+
+
 /** \fn int pds_neuron_connect_input(PdsNeuron *Neuron,PdsSnNatural id,PdsNeuron *NeuronX)
  *  \brief Conecta el valor de salida Y de la Neurona NeuronX con la entrada X[id]
  *  de la neurona Neuron.
@@ -321,27 +428,6 @@ int pds_neuron_connect_input(PdsNeuron *Neuron,PdsSnNatural id,PdsNeuron *Neuron
 }
 
 
-/** \fn int pds_neuron_connect_input_data(PdsNeuron *Neuron,PdsSnNatural id,PdsNInput *NInput)
- *  \brief Conecta el valor de salida Y de la neurona de entrada NInput con la 
- *  entrada X[id] de la neurona Neuron.
- *  \param[in,out] Neuron La neurona a trabajar.
- *  \param[in] id La posición a conectar, id tiene que ser menor que Nel.
- *  \param[in] NInput La neurona de entrada que se conectará a la entrada X[id] de Neuron.
- *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
- *  \ingroup PdsNeuronGroup
- */
-int pds_neuron_connect_input_data(PdsNeuron *Neuron,PdsSnNatural id,PdsNInput *NInput)
-{
-	if(Neuron==NULL)		return FALSE;
-	if(NInput==NULL)		return FALSE;
-	if((id>=Neuron->Nel)||(id<0))	return FALSE;
-
-	Neuron->X[id]=NInput->Y;
-
-	return TRUE;
-}
-
-
 /** \fn int pds_neuron_evaluate_theta(PdsNeuron *Neuron)
  *  \brief Evalua la variable theta de la Neurona Neuron.
  *  \f[ \theta=\sum_{i=0}^{N_{el}-1}{W[i]X[i]} - U \f].
@@ -369,61 +455,14 @@ int pds_neuron_evaluate_theta(PdsNeuron *Neuron)
 }
 
 
-/** \fn PdsSnReal pds_neuron_func(PdsSnReal x)
- *  \brief Función de activación de la neurona de tipo PdsNeuron.
- *  \f[ f(x)=\frac{2}{1+e^{-x}} -1 \f].
- *  \param[in] x Parámetro a evaluar.
- *  \return El resultado de la función. 
- *  \ingroup PdsNeuronGroup
- */
-PdsSnReal pds_neuron_func(PdsSnReal x)
-{
-	return (2.0/(1.0+exp(-x)))-1.0;
-}
-
-
-/** \fn PdsSnReal pds_neuron_dfunc(PdsSnReal x)
- *  \brief Derivada de la función de activación de la neurona de tipo PdsNeuron.
- *  \f[ \frac{\partial  f(x)}{\partial x}=\frac{1-f(x)^2}{2} \f].
- *  \param[in] x Parámetro a evaluar.
- *  \return El resultado de la derivada de la función. 
- *  \ingroup PdsNeuronGroup
- */
-PdsSnReal pds_neuron_dfunc(PdsSnReal x)
-{
-	PdsSnReal m;
-
-	m=(2.0/(1.0+exp(-x)))-1.0;
-
-	return 0.5-0.5*m*m;
-}
-
-
-/** \fn PdsSnReal pds_neuron_invdfunc(PdsSnReal x)
- *  \brief Uno sobre la derivada de la función de activación de la neurona de 
- *  tipo PdsNeuron.
- *  \f[ ( \frac{\partial  f(x)}{\partial x} )^{-1}=\frac{2}{1-f(x)^2} \f].
- *  \param[in] x Parámetro a evaluar.
- *  \return El resultado de uno sobre la derivada de la función. 
- *  \ingroup PdsNeuronGroup
- */
-PdsSnReal pds_neuron_invdfunc(PdsSnReal x)
-{
-	PdsSnReal m;
-
-	m=(2.0/(1.0+exp(-x)))-1.0;
-
-	return 0.5-0.5*m*m;
-}
-
-/** \fn int pds_neuron_evaluate_f(PdsNeuron *Neuron)
+/** \fn int pds_neuron_evaluate_func(PdsNeuron *Neuron)
  *  \brief Evalua la funcion de activación de la Neurona Neuron.
- *  \f[ f(\theta)=\frac{2}{1+e^{-\frac{\theta}{\sigma}}} -1 \f].
+ *  \f[ func\left(\frac{\theta}{\sigma}\right) \f]
  *  \param[in,out] Neuron La neurona a evaluar.
  *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
  *  \ingroup PdsNeuronGroup
  */
-int pds_neuron_evaluate_f(PdsNeuron *Neuron)
+int pds_neuron_evaluate_func(PdsNeuron *Neuron)
 {
 	PdsSnReal x;
 
@@ -431,7 +470,7 @@ int pds_neuron_evaluate_f(PdsNeuron *Neuron)
 
 	x=Neuron->Theta/Neuron->Sigma;
 
-	Neuron->F=pds_neuron_func(x);
+	Neuron->F=Neuron->func(x);
 
 	return TRUE;
 }
@@ -439,8 +478,8 @@ int pds_neuron_evaluate_f(PdsNeuron *Neuron)
 
 /** \fn int pds_neuron_evaluate(PdsNeuron *Neuron)
  *  \brief Evalua la Neurona Neuron.
- *  \f[ \theta=\sum_{i=0}^{N_{el}-1}{W[i]X[i]} - U \f].
- *  \f[ f(\theta)=\frac{2}{1+e^{-\frac{\theta}{\sigma}}} -1 \f].
+ *  \f[ \theta=\sum_{i=0}^{N_{el}-1}{W[i]X[i]} - U \f]
+ *  \f[ func\left(\frac{\theta}{\sigma}\right) \f]
  *  \param[in,out] Neuron La neurona a evaluar.
  *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
  *  \ingroup PdsNeuronGroup
@@ -452,7 +491,7 @@ int pds_neuron_evaluate(PdsNeuron *Neuron)
 
 	m=pds_neuron_evaluate_theta(Neuron);
 	if(m==FALSE)	return FALSE;
-	m=pds_neuron_evaluate_f(Neuron);
+	m=pds_neuron_evaluate_func(Neuron);
 	if(m==FALSE)	return FALSE;
 
 	return TRUE;
@@ -461,7 +500,9 @@ int pds_neuron_evaluate(PdsNeuron *Neuron)
 
 /** \fn int pds_neuron_update(PdsNeuron *Neuron)
  *  \brief Actualiza el valor a la salida de la Neurona Neuron.
- *  \f[ Y=f(\theta) \f].
+ *  \f[ Y=func\left(\frac{\theta}{\sigma}\right) \f]
+ *  Adicionalmente (Neuron->Y[1]=0)
+ *  \f[ e=0 \f]
  *  \param[in,out] Neuron La neurona a trabajar.
  *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
  *  \ingroup PdsNeuronGroup
@@ -479,9 +520,11 @@ int pds_neuron_update(PdsNeuron *Neuron)
 
 /** \fn int pds_neuron_iterate(PdsNeuron *Neuron)
  *  \brief Itera la Neurona Neuron.
- *  \f[ \theta=\sum_{i=0}^{N_{el}-1}{W[i]X[i]} - U \f].
- *  \f[ f(\theta)=\frac{2}{1+e^{-\frac{\theta}{\sigma}}} -1 \f].
- *  \f[ Y=f(\theta) \f].
+ *  \f[ \theta=\sum_{i=0}^{N_{el}-1}{W[i]X[i]} - U \f]
+ *  \f[ func\left(\frac{\theta}{\sigma}\right) \f]
+ *  \f[ Y=func\left(\frac{\theta}{\sigma}\right) \f]
+ *  Adicionalmente (Neuron->Y[1]=0)
+ *  \f[ e=0 \f]
  *  \param[in,out] Neuron La neurona a iterar.
  *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
  *  \ingroup PdsNeuronGroup
@@ -494,7 +537,7 @@ int pds_neuron_iterate(PdsNeuron *Neuron)
 
 	m=pds_neuron_evaluate_theta(Neuron);
 	if(m==FALSE)	return FALSE;
-	m=pds_neuron_evaluate_f(Neuron);
+	m=pds_neuron_evaluate_func(Neuron);
 	if(m==FALSE)	return FALSE;
 
 	*(Neuron->Y)=Neuron->F;
@@ -504,16 +547,16 @@ int pds_neuron_iterate(PdsNeuron *Neuron)
 }
 
 
-/** \fn int pds_neuron_evaluate_e(PdsNeuron *Neuron,PdsSnReal y)
- *  \brief Compara el valor de salida de la neurona con "y" y luego lo carga en
- *  la variable Y[1] de la neurona. 
- *  Neuron->Y[1]=y-Neuron->Y[0]
+/** \fn int pds_neuron_evaluate_error(PdsNeuron *Neuron,PdsSnReal y)
+ *  \brief Compara el valor de salida \f$Y\f$ de la neurona, con "y", luego lo carga
+ *  en la variable Y[1] de la neurona (Neuron->Y[1]=y-Neuron->Y[0]).
+ *  \f[ e= y - Y\f]
  *  \param[in,out] Neuron La neurona a evaluar.
  *  \param[in] y Valor a comparar.
  *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
  *  \ingroup PdsNeuronGroup
  */
-int pds_neuron_evaluate_e(PdsNeuron *Neuron,PdsSnReal y)
+int pds_neuron_evaluate_error(PdsNeuron *Neuron,PdsSnReal y)
 {
 
 	if(Neuron==NULL)		return FALSE;
@@ -526,23 +569,42 @@ int pds_neuron_evaluate_e(PdsNeuron *Neuron,PdsSnReal y)
 
 /** \fn int pds_neuron_backpropagate_error(PdsNeuron *Neuron)
  *  \brief Propaga el error de la salida de la neurona hacia las neuronas conectadas
- * a sus entradas. E_input (i)=W[i] E_salida
+ *  a sus entradas. <br>
+ *  <center>
+ *  \image html backpropagation.png "Backpropagation"
+ *  </center>
+ *  <br>
+ *  <center>
+ *  \image html backpropagation2.png "Backpropagation"
+ *  </center>
+ *  <br>
+ *  \f[ e = e_{\{Y\}} \f]
+ *  \f[ e_i = e_{\{X[i]\}} \f]
+ *  \f[ e_i \leftarrow e_i + \frac{func'\left(\frac{\theta}{\sigma}\right)}{\sigma}W[i]e\f]
+ *  "e" es el error de la salida de la neurona Neuron :Neuron->Y[1].<br>
+ *  "func" es la función de activación da neurona Neuron.<br>
  *  \param[in,out] Neuron La neurona a retropropagar el error.
  *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
  *  \ingroup PdsNeuronGroup
  */
 int pds_neuron_backpropagate_error(PdsNeuron *Neuron)
 {
-	PdsSnReal e;
+	PdsSnReal e,Df;
 	PdsSnNatural i;
+	PdsSnReal *y1;
 
 	if(Neuron==NULL)		return FALSE;
 
+	Df=Neuron->dfunc(Neuron->Theta/Neuron->Sigma)/Neuron->Sigma;
+
 	e=Neuron->Y[1];
 	for(i=0;i<Neuron->Nel;i++)	
 	{
 		if(Neuron->X[i]!=NULL)
-		(*(Neuron->X[i]+1))=(*(Neuron->X[i]+1))+Neuron->W[i]*e;
+        {
+            y1=(Neuron->X[i]+1);
+		    (*y1)=(*y1)+Neuron->W[i]*Df*e;
+        }
 	}
 
 	return TRUE;
@@ -550,10 +612,13 @@ int pds_neuron_backpropagate_error(PdsNeuron *Neuron)
 
 
 /** \fn int pds_neuron_update_weight(PdsNeuron *Neuron, PdsSnReal Alpha)
- *  \brief Actualiza los pesos W[i] de la neurona.
- *  \f[ \triangle W[i]= \alpha e f'(\theta) X[i] \f].
- *  "e" es el error de la salida da neurona Neuron :Neuron->Y[1].
- *  "f" es la función de activación da neurona Neuron.
+ *  \brief Actualiza los pesos W[i] de la neurona (using the Delta rule).
+ *  \f[ e = r-Y\f]
+ *  \f[ \triangle W[i]= \alpha \frac{func'\left(\frac{\theta}{\sigma}\right)}{\sigma} X[i] e\f]
+ *  "e" es el error de la salida de la neurona Neuron :Neuron->Y[1].<br>
+ *  "Y" salida de la neurona Neuron :Neuron->Y[0].<br>
+ *  "func" es la función de activación da neurona Neuron.<br>
+ *  "X" Vector con los valores de todas las entradas de la neurona.<br>
  *  \param[in,out] Neuron La neurona a actualizar los pesos.
  *  \param[in] Alpha Factor de aprendizaje de los pesos.
  *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
@@ -566,7 +631,7 @@ int pds_neuron_update_weight(PdsNeuron *Neuron, PdsSnReal Alpha)
 
 	if(Neuron==NULL)		return FALSE;
 
-	Df=pds_neuron_dfunc(Neuron->Theta);
+	Df=Neuron->dfunc(Neuron->Theta/Neuron->Sigma)/Neuron->Sigma;
 	e=Neuron->Y[1];
 
 	for(i=0;i<Neuron->Nel;i++)	
@@ -603,45 +668,15 @@ int pds_neuron_get_xtx(PdsNeuron *Neuron,PdsSnReal *XtX)
 	return TRUE;
 }
 
-/** \fn int pds_neuron_update_weight_norm(PdsNeuron *Neuron, PdsSnReal Alpha,PdsSnReal XtX)
- *  \brief Actualiza los pesos W[i] de la neurona.
- *  \f[ \triangle W[i]= \alpha e f'(\theta) \frac{X[i]}{X^TX} \f].
- *  "e" es el error de la salida da neurona Neuron :Neuron->Y[1].
- *  "f" es la función de activación da neurona Neuron.
- *  \param[in,out] Neuron La neurona a actualizar los pesos.
- *  \param[in] Alpha Factor de aprendizaje de los pesos.
- *  \param[in] XtX Suma cuadrática de los valores de entrada X[i].
- *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
- *  \ingroup PdsNeuronGroup
- */
-int pds_neuron_update_weight_norm(PdsNeuron *Neuron, PdsSnReal Alpha,PdsSnReal XtX)
-{
-	PdsSnReal e,x,Df;
-	PdsSnNatural i;
-
-	if(Neuron==NULL)		return FALSE;
-
-	Df=pds_neuron_dfunc(Neuron->Theta);
-	e=Neuron->Y[1];
-
-	for(i=0;i<Neuron->Nel;i++)	
-	{
-		if((Neuron->X[i]!=NULL)&&(XtX>0))
-		{
-			x=(*(Neuron->X[i]));
-			Neuron->W[i]=Neuron->W[i]+Alpha*e*Df*x/XtX;
-		}
-	}
-
-	return TRUE;
-}
-
 
 /** \fn int pds_neuron_update_weight_normalized(PdsNeuron *Neuron, PdsSnReal Alpha)
- *  \brief Actualiza los pesos W[i] de la neurona.
- *  \f[ \triangle W[i]= \alpha e f'(\theta) \frac{X[i]}{X^TX} \f].
- *  "e" es el error de la salida da neurona Neuron :Neuron->Y[1].
- *  "f" es la función de activación da neurona Neuron.
+ *  \brief Actualiza los pesos W[i] de la neurona (using the Delta rule).
+ *  \f[ e = r-Y\f]
+ *  \f[ \triangle W[i]= \alpha \frac{func'\left(\frac{\theta}{\sigma}\right)}{\sigma} \frac{X[i]}{X^TX} e \f]
+ *  "e" es el error de la salida de la neurona Neuron :Neuron->Y[1].<br>
+ *  "Y" salida de la neurona Neuron :Neuron->Y[0].<br>
+ *  "func" es la función de activación da neurona Neuron.<br>
+ *  "X" Vector con los valores de todas las entradas de la neurona.<br>
  *  \param[in,out] Neuron La neurona a actualizar los pesos.
  *  \param[in] Alpha Factor de aprendizaje de los pesos.
  *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
@@ -654,7 +689,7 @@ int pds_neuron_update_weight_normalized(PdsNeuron *Neuron, PdsSnReal Alpha)
 
 	if(Neuron==NULL)		return FALSE;
 
-	Df=pds_neuron_dfunc(Neuron->Theta);
+	Df=Neuron->dfunc(Neuron->Theta/Neuron->Sigma)/Neuron->Sigma;
 	e=Neuron->Y[1];
 	
 	for(XtX=0,i=0;i<Neuron->Nel;i++)	
@@ -674,7 +709,7 @@ int pds_neuron_update_weight_normalized(PdsNeuron *Neuron, PdsSnReal Alpha)
 
 
 /** \fn int pds_neuron_fprintf(const PdsNeuron *Neuron, FILE *fd)
- *  \brief Guarda en un archivo de texto los pesos W[i] y el valor de U.
+ *  \brief Guarda en un archivo de texto los pesos {W[i]} y  {U, Sigma}.
  *  Ocupando una linea cada uno, y separando los elementos por un TAB.
  *  \param[in] Neuron La neurona a leer.
  *  \param[in,out] fd Manejador del fichero a escribir.
@@ -693,7 +728,7 @@ int pds_neuron_fprintf(const PdsNeuron *Neuron, FILE *fd)
 	for(i=0;i<Neuron->Nel;i++)	
 	{
 		if(i!=(Neuron->Nel-1))	tmp=fprintf(fd,"%e\t",Neuron->W[i]);
-		else			tmp=fprintf(fd,"%e\n",Neuron->W[i]);
+		else                    tmp=fprintf(fd,"%e\n",Neuron->W[i]);
 	}
 
 	tmp=fprintf(fd,"%e\t",Neuron->U);
@@ -704,7 +739,7 @@ int pds_neuron_fprintf(const PdsNeuron *Neuron, FILE *fd)
 
 
 /** \fn int pds_neuron_fscanf(PdsNeuron *Neuron, FILE *fd)
- *  \brief Lee de un archivo de texto los pesos W[i] y el valor de U.
+ *  \brief Lee de un archivo de texto los pesos {W[i]} y  {U, Sigma}.
  *  Ocupando una linea cada uno, y separando los elementos por un TAB.
  *  \param[out] Neuron La neurona a escribir.
  *  \param[in,out] fd Manejador del fichero a leer.
@@ -737,7 +772,7 @@ int pds_neuron_fscanf(PdsNeuron *Neuron, FILE *fd)
 
 
 /** \fn int pds_neuron_fwrite(const PdsNeuron *Neuron, FILE *fd)
- *  \brief Guarda en un archivo binario los pesos W[i] y el valor de U.
+ *  \brief Guarda en un archivo binario los pesos W[i], el valor de U y Sigma.
  *  \param[in] Neuron La neurona a leer.
  *  \param[in,out] fd Manejador del fichero binario a escribir.
  *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL o fd==NULL). 
@@ -769,7 +804,7 @@ int pds_neuron_fwrite(const PdsNeuron *Neuron, FILE *fd)
 
 
 /** \fn int pds_neuron_fread(PdsNeuron *Neuron, FILE *fd)
- *  \brief Lee de un archivo binario los pesos W[i] y el valor de U.
+ *  \brief Lee de un archivo binario los pesos W[i], el valor de U y Sigma.
  *  \param[out] Neuron La neurona a escribir.
  *  \param[in,out] fd Manejador del fichero binario a leer.
  *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL o fd==NULL). 
