@@ -39,6 +39,7 @@ extern "C" {
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <pds/pdsninput.h>
 #include <pds/pdssnglobal.h>
 
 #ifndef TRUE
@@ -49,25 +50,25 @@ extern "C" {
 	#define FALSE 0
 #endif
 
-/** \defgroup PdsNeuronGroup Funciones del módulo PdsNeuron.
+/** \defgroup PdsNeuronGroup Módulo PdsNeuron.
  *  \brief Funciones que trabajan con neuronas.
  *  
  *  <br>Estas funciones trabajan con una neurona de la forma.<br>
+ *  <center>
  *  \image html PdsNeuron.png "Neurona de n=Nel entradas."
+ *  </center>
  * @{
  */
 
-/*! \def _PNSETSW(Neuron,i,m)
- *  Es equivalente a Neuron->W[i]=m.
- *  \ingroup PdsNeuronGroup
-*/
-#define _PNSETSW(Neuron,i,m) pds_neuron_set_synaptic_weight(Neuron,i,m)
 
 
 /*! \struct PdsNeuron
  *  \brief La estructura tipo  PdsNeuron .
  *  Esta estructura genera una neurona de Nel entradas.
+ *
+ *  <center>
  *  \image html PdsNeuron.png "Neurona de n=Nel entradas."
+ *  </center>
  *  Para usar incluir pds/pdssn.h.
  *  \ingroup PdsNeuronGroup
  *  \author Fernando Pujaico Rivera
@@ -76,16 +77,28 @@ typedef struct
 {
 	/*! Un arreglo de punteros con Nel elementos. */
 	PdsSnReal **X;
+
 	/*! Un arreglo de Nel elementos. */
 	PdsSnReal *W;
+
 	/*! Número de elementos. */
 	PdsSnNatural Nel;
+
 	/*! Valor de umbral. */
 	PdsSnReal U;
+
 	/*! Valor de Sigma. */
 	PdsSnReal Sigma;
+
 	/*! Valor de ponderacion. */
 	PdsSnReal Theta;
+
+	/*! Función de activación.*/
+    double (*func)(double);
+
+	/*! Derivada de la función de activación.*/
+    double (*dfunc)(double);
+
 	/*! Valor de activación. */
 	PdsSnReal F;
 	/*! Valor de salida *Y, y un espacio extra Y[1]. */
@@ -93,16 +106,111 @@ typedef struct
 }PdsNeuron;
 
 
+
+
+/** @name pds_neuron_new
+ *  Creando nuevas neuronas
+ *
+ *  <center>
+ *  \image html PdsNeuron.png "Neurona de n=Nel entradas."
+ *  </center>
+ *  <br>
+ * @{
+ */
+
+
 /** \fn PdsNeuron *pds_neuron_new(PdsSnNatural Nel)
- *  \brief Crea un vector de tipo PdsNeuron e inicia con cero todos los elementos.
- *  \param[in] Nel Es el número de elementos del vector.
+ *  \brief Crea una neurona de tipo PdsNeuron de Nel entradas.
+ *
+ *  Los valores por defecto son :
+ *  \f[ U=1.0 \f]
+ *  \f[ \sigma=1.0 \f]
+ *  \f[ func(x)=tanh\left(\frac{x}{2}\right) \f]
+ *  \param[in] Nel Es el número de entradas de la neurona.
  *  \return Un puntero al vector de tipo PdsNeuron.
  *  \ingroup PdsNeuronGroup
  */
 PdsNeuron *pds_neuron_new(PdsSnNatural Nel);
 
 
-/** \fn int pds_neuron_get_synaptic_weight(const PdsNeuron *Neuron, PdsSnNatural x, PdsSnReal *m)
+/** \fn PdsNeuron *pds_neuron_new_with_params(PdsSnNatural Nel,PdsSnReal W,PdsSnReal U,PdsSnReal Sigma,double (*func)(double),double (*dfunc)(double));
+ *  \brief Crea una neurona de tipo PdsNeuron de Nel entradas.
+ *
+ *  \param[in] Nel Es el número de entradas de la neurona.
+ *  \param[in] W Peso inicial de todas las entradas
+ *  \param[in] U Umbral de disparo.
+ *  \param[in] Sigma Factor de escala de la función de activación.
+ *  \param[in] func Función de activación.
+ *  \param[in] dfunc Derivada de la función de activación.
+ *  \return Un puntero al vector de tipo PdsNeuron.
+ *  \ingroup PdsNeuronGroup
+ */
+PdsNeuron *pds_neuron_new_with_params(PdsSnNatural Nel,PdsSnReal W,PdsSnReal U,PdsSnReal Sigma,double (*func)(double),double (*dfunc)(double));
+
+//@}
+
+/** @name pds_neuron_<input_methods>
+ *  Obteniendo datos.
+ *
+ * @{
+ */
+
+
+/** \fn int pds_neuron_connect_input(PdsNeuron *Neuron,PdsSnNatural id,PdsNeuron *NeuronX)
+ *  \brief Conecta el valor de salida Y de la Neurona NeuronX con la entrada X[id]
+ *  de la neurona Neuron.
+ *  \param[in,out] Neuron La neurona a trabajar.
+ *  \param[in] id La posición a conectar, id tiene que ser menor que Nel.
+ *  \param[in] NeuronX Neurona que se conectará a la entrada X[id] de Neuron.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
+ *  \ingroup PdsNeuronGroup
+ */
+int pds_neuron_connect_input(PdsNeuron *Neuron,PdsSnNatural id,PdsNeuron *NeuronX);
+
+
+/** \fn const PdsSnReal *pds_neuron_get_dendrite(const PdsNeuron *Neuron,PdsSnNatural id)
+ *  \brief Devuelve la dirección de la entrada X[id] de la Neurona.
+ *  \param[in] Neuron La neurona en consulta.
+ *  \param[in] id La posición a consultar, id tiene que ser menor que Nel.
+ *  \return La dirección de la entrada X[id] o NULL en caso de error (ej: Neuron==NULL). 
+ *  \ingroup PdsNeuronGroup
+ */
+const PdsSnReal *pds_neuron_get_dendrite(const PdsNeuron *Neuron,PdsSnNatural id);
+
+
+/** \fn int pds_neuron_get_input(const PdsNeuron *Neuron,PdsSnNatural id,PdsSnReal *m)
+ *  \brief Devuelve el valor de la entrada X[id] de la Neurona.
+ *  \param[in] Neuron La neurona en consulta.
+ *  \param[in] id La posición a consultar, id tiene que ser menor que Nel.
+ *  \param[out] m Donde se cargará el valor de la entrada X[id].
+ *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
+ *  \ingroup PdsNeuronGroup
+ */
+int pds_neuron_get_input(const PdsNeuron *Neuron,PdsSnNatural id,PdsSnReal *m);
+
+
+
+/** \fn int pds_neuron_connect_input_with_ninput(PdsNeuron *Neuron,PdsSnNatural id,PdsNInput *NInput)
+ *  \brief Conecta el valor de salida Y de la neurona de entrada NInput con la 
+ *  entrada X[id] de la neurona Neuron.
+ *  \param[in,out] Neuron La neurona a trabajar.
+ *  \param[in] id La posición a conectar, id tiene que ser menor que Nel.
+ *  \param[in] NInput La neurona de entrada que se conectará a la entrada X[id] de Neuron.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
+ *  \ingroup PdsNeuronGroup
+ */
+int pds_neuron_connect_input_with_ninput(PdsNeuron *Neuron,PdsSnNatural id,PdsNInput *NInput);
+
+
+//@}
+
+/** @name pds_neuron_<weight_methods>
+ *  Trabajando con el synaptic_weight
+ *
+ * @{
+ */
+
+/** \fn int pds_neuron_get_weight(const PdsNeuron *Neuron, PdsSnNatural x, PdsSnReal *m)
  *  \brief Devuelve el valor en la posición (x) del vector de pesos de la Neurona.
  *  (x) inicia con (0).
  *  \param[in] Neuron La neurona en consulta.
@@ -112,10 +220,15 @@ PdsNeuron *pds_neuron_new(PdsSnNatural Nel);
  *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
  *  \ingroup PdsNeuronGroup
  */
-int pds_neuron_get_synaptic_weight(const PdsNeuron *Neuron, PdsSnNatural x, PdsSnReal *m);
+int pds_neuron_get_weight(const PdsNeuron *Neuron, PdsSnNatural x, PdsSnReal *m);
 
+/*! \def PDS_NSETW(Neuron,i,m)
+ *  Es equivalente a Neuron->W[i]=m.
+ *  \ingroup PdsNeuronGroup
+*/
+#define PDS_NSETW(Neuron,i,m) pds_neuron_set_weight(Neuron,i,m)
 
-/** \fn int pds_neuron_set_synaptic_weight(PdsNeuron *Neuron, PdsSnNatural x, PdsSnReal m)
+/** \fn int pds_neuron_set_weight(PdsNeuron *Neuron, PdsSnNatural x, PdsSnReal m)
  *  \brief Escribe el valor m en la posición (x) del vector de pesos de la Neurona.
  *  (x) inicia con (0).
  *  \param[in,out] Neuron La neurona a escribir.
@@ -125,17 +238,29 @@ int pds_neuron_get_synaptic_weight(const PdsNeuron *Neuron, PdsSnNatural x, PdsS
  *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
  *  \ingroup PdsNeuronGroup
  */
-int pds_neuron_set_synaptic_weight(PdsNeuron *Neuron, PdsSnNatural x, PdsSnReal m);
+int pds_neuron_set_weight(PdsNeuron *Neuron, PdsSnNatural x, PdsSnReal m);
 
 
-/** \fn int pds_neuron_init_synaptic_weight(PdsNeuron *Neuron, PdsSnReal m)
+/** \fn int pds_neuron_init_weight(PdsNeuron *Neuron, PdsSnReal m)
  *  \brief Inicia el vector de pesos de la Neurona.
  *  \param[in,out] Neuron La neurona a escribir.
  *  \param[in] m El valor de los pesos de la neurona.
  *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
  *  \ingroup PdsNeuronGroup
  */
-int pds_neuron_init_synaptic_weight(PdsNeuron *Neuron, PdsSnReal m);
+int pds_neuron_init_weight(PdsNeuron *Neuron, PdsSnReal m);
+
+
+/** \fn int pds_neuron_init_weight_rand(PdsNeuron *Neuron, PdsSnReal min, PdsSnReal max)
+ *  \brief Inicia el vector de pesos de la Neurona con valores aleatorios
+ *  uniformemente distribuido entre min y max.
+ *  \param[in,out] Neuron La neurona a escribir.
+ *  \param[in] min El minimo valor encontrado en los pesos de la neurona.
+ *  \param[in] max El minimo valor encontrado en los pesos de la neurona.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
+ *  \ingroup PdsNeuronGroup
+ */
+int pds_neuron_init_weight_rand(PdsNeuron *Neuron, PdsSnReal min, PdsSnReal max);
 
 
 /** \fn int pds_neuron_get_number_of_inputs(const PdsNeuron *Neuron,PdsSnNatural *Nelements)
@@ -147,26 +272,24 @@ int pds_neuron_init_synaptic_weight(PdsNeuron *Neuron, PdsSnReal m);
  */
 int pds_neuron_get_number_of_inputs(const PdsNeuron *Neuron,PdsSnNatural *Nelements);
 
+//@}
 
-/** \fn int pds_neuron_get_sigma(const PdsNeuron *Neuron,PdsSnReal *Sigma)
- *  \brief Devuelve el valor de  sigma de la Neurona.
- *  \param[in] Neuron La neurona en consulta.
- *  \param[out] Sigma En donde se guardará el valor de sigma de la neurona.
+/** @name pds_neuron_<theta_methods>
+ *  Obteniendo datos.
+ *
+ * @{
+ */
+
+/** \fn int pds_neuron_evaluate_theta(PdsNeuron *Neuron)
+ *  \brief Evalua la variable theta de la Neurona Neuron.
+ *  \f[ \theta=\sum_{i=0}^{N_{el}-1}{W[i]X[i]} - U \f].
+ *  \param[in,out] Neuron La neurona a evaluar.
  *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
  *  \ingroup PdsNeuronGroup
  */
-int pds_neuron_get_sigma(const PdsNeuron *Neuron,PdsSnReal *Sigma);
+int pds_neuron_evaluate_theta(PdsNeuron *Neuron);
 
 
-/** \fn int pds_neuron_set_sigma(PdsNeuron *Neuron,PdsSnReal Sigma)
- *  \brief Coloca el valor de  sigma de la Neurona.
- *  \param[in,out] Neuron La neurona a escribir.
- *  \param[in] Sigma El valor de sigma de la neurona.
- *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
- *  \ingroup PdsNeuronGroup
- */
-int pds_neuron_set_sigma(PdsNeuron *Neuron,PdsSnReal Sigma);
-
 
 /** \fn int pds_neuron_get_u(const PdsNeuron *Neuron,PdsSnReal *U)
  *  \brief Devuelve el valor de  U de la Neurona.
@@ -187,27 +310,96 @@ int pds_neuron_get_u(const PdsNeuron *Neuron,PdsSnReal *U);
  */
 int pds_neuron_set_u(PdsNeuron *Neuron,PdsSnReal U);
 
+//@}
 
-/** \fn PdsSnReal *pds_neuron_get_dendrite(const PdsNeuron *Neuron,PdsSnNatural id)
- *  \brief Devuelve la dirección de la entrada X[id] de la Neurona.
- *  \param[in] Neuron La neurona en consulta.
- *  \param[in] id La posición a consultar, id tiene que ser menor que Nel.
- *  \return La dirección de la entrada X[id] o NULL en caso de error (ej: Neuron==NULL). 
- *  \ingroup PdsNeuronGroup
+
+/** @name pds_neuron_<func_methods>
+ *  Trabajando con la función de activación.
+ *
+ * @{
  */
-PdsSnReal *pds_neuron_get_dendrite(const PdsNeuron *Neuron,PdsSnNatural id);
 
-
-/** \fn int pds_neuron_get_input(const PdsNeuron *Neuron,PdsSnNatural id,PdsSnReal *m)
- *  \brief Devuelve el valor de la entrada X[id] de la Neurona.
- *  \param[in] Neuron La neurona en consulta.
- *  \param[in] id La posición a consultar, id tiene que ser menor que Nel.
- *  \param[out] m Donde se cargará el valor de la entrada X[id].
+/** \fn int pds_neuron_evaluate_func(PdsNeuron *Neuron)
+ *  \brief Evalua la funcion de activación de la Neurona Neuron.
+ *  \f[ func\left(\frac{\theta}{\sigma}\right) \f]
+ *  \param[in,out] Neuron La neurona a evaluar.
  *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
  *  \ingroup PdsNeuronGroup
  */
-int pds_neuron_get_input(const PdsNeuron *Neuron,PdsSnNatural id,PdsSnReal *m);
+int pds_neuron_evaluate_func(PdsNeuron *Neuron);
 
+/** \fn int pds_neuron_get_sigma(const PdsNeuron *Neuron,PdsSnReal *Sigma)
+ *  \brief Devuelve el valor de  sigma de la Neurona.
+ *  \param[in] Neuron La neurona en consulta.
+ *  \param[out] Sigma En donde se guardará el valor de sigma de la neurona.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
+ *  \ingroup PdsNeuronGroup
+ */
+int pds_neuron_get_sigma(const PdsNeuron *Neuron,PdsSnReal *Sigma);
+
+
+/** \fn int pds_neuron_set_sigma(PdsNeuron *Neuron,PdsSnReal Sigma)
+ *  \brief Coloca el valor de  sigma de la Neurona.
+ *  \param[in,out] Neuron La neurona a escribir.
+ *  \param[in] Sigma El valor de sigma de la neurona.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
+ *  \ingroup PdsNeuronGroup
+ */
+int pds_neuron_set_sigma(PdsNeuron *Neuron,PdsSnReal Sigma);
+
+//@}
+
+
+/** @name pds_neuron_<evaluate>
+ *  Trabajando con la 
+ *
+ * @{
+ */
+
+/** \fn int pds_neuron_evaluate(PdsNeuron *Neuron)
+ *  \brief Evalua la Neurona Neuron.
+ *  \f[ \theta=\sum_{i=0}^{N_{el}-1}{W[i]X[i]} - U \f]
+ *  \f[ func\left(\frac{\theta}{\sigma}\right) \f]
+ *  \param[in,out] Neuron La neurona a evaluar.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
+ *  \ingroup PdsNeuronGroup
+ */
+int pds_neuron_evaluate(PdsNeuron *Neuron);
+
+
+/** \fn int pds_neuron_update(PdsNeuron *Neuron)
+ *  \brief Actualiza el valor a la salida de la Neurona Neuron.
+ *  \f[ Y=func\left(\frac{\theta}{\sigma}\right) \f]
+ *  Adicionalmente (Neuron->Y[1]=0)
+ *  \f[ e=0 \f].
+ *  \param[in,out] Neuron La neurona a trabajar.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
+ *  \ingroup PdsNeuronGroup
+ */
+int pds_neuron_update(PdsNeuron *Neuron);
+
+
+/** \fn int pds_neuron_iterate(PdsNeuron *Neuron)
+ *  \brief Itera la Neurona Neuron.
+ *  \f[ \theta=\sum_{i=0}^{N_{el}-1}{W[i]X[i]} - U \f]
+ *  \f[ func\left(\frac{\theta}{\sigma}\right) \f]
+ *  \f[ Y=func\left(\frac{\theta}{\sigma}\right) \f]
+ *  Adicionalmente (Neuron->Y[1]=0)
+ *  \f[ e=0 \f]
+ *  \param[in,out] Neuron La neurona a iterar.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
+ *  \ingroup PdsNeuronGroup
+ */
+int pds_neuron_iterate(PdsNeuron *Neuron);
+
+
+//@}
+
+/** @name pds_neuron_<output_methods>
+ *  Obteniendo datos.
+ *
+ * @{
+ */
 
 /** \fn const PdsSnReal *pds_neuron_get_axon(const PdsNeuron *Neuron)
  *  \brief Devuelve la dirección del axon (salida) de la Neurona.
@@ -228,141 +420,70 @@ const PdsSnReal *pds_neuron_get_axon(const PdsNeuron *Neuron);
 int pds_neuron_get_output(const PdsNeuron *Neuron,PdsSnReal *m);
 
 
-/** \fn int pds_neuron_connect_input(PdsNeuron *Neuron,PdsSnNatural id,PdsNeuron *NeuronX)
- *  \brief Conecta el valor de salida Y de la Neurona NeuronX con la entrada X[id]
- *  de la neurona Neuron.
- *  \param[in,out] Neuron La neurona a trabajar.
- *  \param[in] id La posición a conectar, id tiene que ser menor que Nel.
- *  \param[in] NeuronX Neurona que se conectará a la entrada X[id] de Neuron.
- *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
- *  \ingroup PdsNeuronGroup
+//@}
+
+
+/** @name pds_neuron_<others_methods>
+ *  Trabajando con la 
+ *
+ * @{
  */
-int pds_neuron_connect_input(PdsNeuron *Neuron,PdsSnNatural id,PdsNeuron *NeuronX);
-
-
-/** \fn int pds_neuron_connect_input_data(PdsNeuron *Neuron,PdsSnNatural id,PdsNInput *NInput)
- *  \brief Conecta el valor de salida Y de la neurona de entrada NInput con la 
- *  entrada X[id] de la neurona Neuron.
- *  \param[in,out] Neuron La neurona a trabajar.
- *  \param[in] id La posición a conectar, id tiene que ser menor que Nel.
- *  \param[in] NInput La neurona de entrada que se conectará a la entrada X[id] de Neuron.
- *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
- *  \ingroup PdsNeuronGroup
- */
-int pds_neuron_connect_input_data(PdsNeuron *Neuron,PdsSnNatural id,PdsNInput *NInput);
-
-
-/** \fn int pds_neuron_evaluate_theta(PdsNeuron *Neuron)
- *  \brief Evalua la variable theta de la Neurona Neuron.
- *  \f[ \theta=\sum_{i=0}^{N_{el}-1}{W[i]X[i]} - U \f].
- *  \param[in,out] Neuron La neurona a evaluar.
- *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
- *  \ingroup PdsNeuronGroup
- */
-int pds_neuron_evaluate_theta(PdsNeuron *Neuron);
-
-
-/** \fn PdsSnReal pds_neuron_func(PdsSnReal x)
- *  \brief Función de activación de la neurona de tipo PdsNeuron.
- *  \f[ f(x)=\frac{2}{1+e^{-x}} -1 \f].
- *  \param[in] x Parámetro a evaluar.
- *  \return El resultado de la función. 
- *  \ingroup PdsNeuronGroup
- */
-PdsSnReal pds_neuron_func(PdsSnReal x);
-
-
-/** \fn PdsSnReal pds_neuron_dfunc(PdsSnReal x)
- *  \brief Derivada de la función de activación de la neurona de tipo PdsNeuron.
- *  \f[ \frac{\partial  f(x)}{\partial x}=\frac{1-f(x)^2}{2} \f].
- *  \param[in] x Parámetro a evaluar.
- *  \return El resultado de la derivada de la función. 
- *  \ingroup PdsNeuronGroup
- */
-PdsSnReal pds_neuron_dfunc(PdsSnReal x);
-
-
-/** \fn PdsSnReal pds_neuron_invdfunc(PdsSnReal x)
- *  \brief Uno sobre la derivada de la función de activación de la neurona de 
- *  tipo PdsNeuron.
- *  \f[ ( \frac{\partial  f(x)}{\partial x} )^{-1}=\frac{2}{1-f(x)^2} \f].
- *  \param[in] x Parámetro a evaluar.
- *  \return El resultado de uno sobre la derivada de la función. 
- *  \ingroup PdsNeuronGroup
- */
-PdsSnReal pds_neuron_invdfunc(PdsSnReal x);
-
-
-/** \fn int pds_neuron_evaluate_f(PdsNeuron *Neuron)
- *  \brief Evalua la funcion de activación de la Neurona Neuron.
- *  \f[ f(\theta)=\frac{2}{1+e^{-\frac{\theta}{\sigma}}} -1 \f].
- *  \param[in,out] Neuron La neurona a evaluar.
- *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
- *  \ingroup PdsNeuronGroup
- */
-int pds_neuron_evaluate_f(PdsNeuron *Neuron);
-
-
-/** \fn int pds_neuron_evaluate(PdsNeuron *Neuron)
- *  \brief Evalua la Neurona Neuron.
- *  \f[ \theta=\sum_{i=0}^{N_{el}-1}{W[i]X[i]} - U \f].
- *  \f[ f(\theta)=\frac{2}{1+e^{-\frac{\theta}{\sigma}}} -1 \f].
- *  \param[in,out] Neuron La neurona a evaluar.
- *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
- *  \ingroup PdsNeuronGroup
- */
-int pds_neuron_evaluate(PdsNeuron *Neuron);
-
-
-/** \fn int pds_neuron_update(PdsNeuron *Neuron)
- *  \brief Actualiza el valor a la salida de la Neurona Neuron.
- *  \f[ Y=f(\theta) \f].
- *  \param[in,out] Neuron La neurona a trabajar.
- *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
- *  \ingroup PdsNeuronGroup
- */
-int pds_neuron_update(PdsNeuron *Neuron);
-
-
-/** \fn int pds_neuron_iterate(PdsNeuron *Neuron)
- *  \brief Itera la Neurona Neuron.
- *  \f[ \theta=\sum_{i=0}^{N_{el}-1}{W[i]X[i]} - U \f].
- *  \f[ f(\theta)=\frac{2}{1+e^{-\frac{\theta}{\sigma}}} -1 \f].
- *  \f[ Y=f(\theta) \f].
- *  \param[in,out] Neuron La neurona a iterar.
- *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
- *  \ingroup PdsNeuronGroup
- */
-int pds_neuron_iterate(PdsNeuron *Neuron);
 
 
 
-/** \fn int pds_neuron_evaluate_e(PdsNeuron *Neuron,PdsSnReal y)
- *  \brief Compara el valor de salida de la neurona con "y" y luego lo carga en
- *  la variable Y[1] de la neurona. 
- *  Neuron->Y[1]=y-Neuron->Y[0]
+/** \fn int pds_neuron_evaluate_error(PdsNeuron *Neuron,PdsSnReal y)
+ *  \brief Compara el valor de salida \f$Y\f$ de la neurona, con "y", luego lo carga
+ *  en la variable Y[1] de la neurona (Neuron->Y[1]=y-Neuron->Y[0]).
+ *  \f[ e= y - Y\f]
  *  \param[in,out] Neuron La neurona a evaluar.
  *  \param[in] y Valor a comparar.
  *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
  *  \ingroup PdsNeuronGroup
  */
-int pds_neuron_evaluate_e(PdsNeuron *Neuron,PdsSnReal y);
+int pds_neuron_evaluate_error(PdsNeuron *Neuron,PdsSnReal y);
 
 
 /** \fn int pds_neuron_backpropagate_error(PdsNeuron *Neuron)
  *  \brief Propaga el error de la salida de la neurona hacia las neuronas conectadas
- * a sus entradas. E_input (i)=W[i] E_salida
+ *  a sus entradas. <br>
+ *  <center>
+ *  \image html backpropagation.png "Backpropagation"
+ *  </center>
+ *  <br>
+ *  <center>
+ *  \image html backpropagation2.png "Backpropagation"
+ *  </center>
+ *  <br>
+ *  \f[ e = e_{\{Y\}} \f]
+ *  \f[ e_i = e_{\{X[i]\}} \f]
+ *  \f[ e_i \leftarrow e_i + \frac{func'\left(\frac{\theta}{\sigma}\right)}{\sigma}W[i]e\f]
+ *  "e" es el error de la salida de la neurona Neuron :Neuron->Y[1].<br>
+ *  "func" es la función de activación da neurona Neuron.<br>
  *  \param[in,out] Neuron La neurona a retropropagar el error.
  *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
  *  \ingroup PdsNeuronGroup
  */
 int pds_neuron_backpropagate_error(PdsNeuron *Neuron);
 
+
+
+
+//@}
+
+/** @name pds_neuron_<update_weight>
+ *  Atualizando peso de las neuronas.
+ *
+ * @{
+ */
+
 /** \fn int pds_neuron_update_weight(PdsNeuron *Neuron, PdsSnReal Alpha)
- *  \brief Actualiza los pesos W[i] de la neurona.
- *  \f[ \triangle W[i]= \alpha e f'(\theta) X[i] \f].
- *  "e" es el error de la salida da neurona Neuron :Neuron->Y[1].
- *  "f" es la función de activación da neurona Neuron.
+ *  \brief Actualiza los pesos W[i] de la neurona (using the Delta rule).
+ *  \f[ e = r-Y\f]
+ *  \f[ \triangle W[i]= \alpha \frac{func'\left(\frac{\theta}{\sigma}\right)}{\sigma} X[i] e\f]
+ *  "e" es el error de la salida de la neurona Neuron :Neuron->Y[1].<br>
+ *  "Y" salida de la neurona Neuron :Neuron->Y[0].<br>
+ *  "func" es la función de activación da neurona Neuron.<br>
+ *  "X" Vector con los valores de todas las entradas de la neurona.<br>
  *  \param[in,out] Neuron La neurona a actualizar los pesos.
  *  \param[in] Alpha Factor de aprendizaje de los pesos.
  *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
@@ -382,25 +503,14 @@ int pds_neuron_update_weight(PdsNeuron *Neuron, PdsSnReal Alpha);
 int pds_neuron_get_xtx(PdsNeuron *Neuron,PdsSnReal *XtX);
 
 
-/** \fn int pds_neuron_update_weight_norm(PdsNeuron *Neuron, PdsSnReal Alpha,PdsSnReal XtX)
- *  \brief Actualiza los pesos W[i] de la neurona.
- *  \f[ \triangle W[i]= \alpha e f'(\theta) \frac{X[i]}{X^TX} \f].
- *  "e" es el error de la salida da neurona Neuron :Neuron->Y[1].
- *  "f" es la función de activación da neurona Neuron.
- *  \param[in,out] Neuron La neurona a actualizar los pesos.
- *  \param[in] Alpha Factor de aprendizaje de los pesos.
- *  \param[in] XtX Suma cuadrática de los valores de entrada X[i].
- *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
- *  \ingroup PdsNeuronGroup
- */
-int pds_neuron_update_weight_norm(PdsNeuron *Neuron, PdsSnReal Alpha,PdsSnReal XtX);
-
-
 /** \fn int pds_neuron_update_weight_normalized(PdsNeuron *Neuron, PdsSnReal Alpha)
- *  \brief Actualiza los pesos W[i] de la neurona.
- *  \f[ \triangle W[i]= \alpha e f'(\theta) \frac{X[i]}{X^TX} \f].
- *  "e" es el error de la salida da neurona Neuron :Neuron->Y[1].
- *  "f" es la función de activación da neurona Neuron.
+ *  \brief Actualiza los pesos W[i] de la neurona (using the Delta rule).
+ *  \f[ e = r-Y\f]
+ *  \f[ \triangle W[i]= \alpha \frac{func'\left(\frac{\theta}{\sigma}\right)}{\sigma} \frac{X[i]}{X^TX} e \f]
+ *  "e" es el error de la salida de la neurona Neuron :Neuron->Y[1].<br>
+ *  "Y" salida de la neurona Neuron :Neuron->Y[0].<br>
+ *  "func" es la función de activación da neurona Neuron.<br>
+ *  "X" Vector con los valores de todas las entradas de la neurona.<br>
  *  \param[in,out] Neuron La neurona a actualizar los pesos.
  *  \param[in] Alpha Factor de aprendizaje de los pesos.
  *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
@@ -408,9 +518,16 @@ int pds_neuron_update_weight_norm(PdsNeuron *Neuron, PdsSnReal Alpha,PdsSnReal X
  */
 int pds_neuron_update_weight_normalized(PdsNeuron *Neuron, PdsSnReal Alpha);
 
+//@}
+
+/** @name pds_neuron_<printf>
+ *  Imprimeindo y leyendo datos.
+ *
+ * @{
+ */
 
 /** \fn int pds_neuron_fprintf(const PdsNeuron *Neuron, FILE *fd)
- *  \brief Guarda en un archivo de texto los pesos W[i] y el valor de U.
+ *  \brief Guarda en un archivo de texto los pesos {W[i]} y  {U, Sigma}.
  *  Ocupando una linea cada uno, y separando los elementos por un TAB.
  *  \param[in] Neuron La neurona a leer.
  *  \param[in,out] fd Manejador del fichero a escribir.
@@ -421,7 +538,7 @@ int pds_neuron_fprintf(const PdsNeuron *Neuron, FILE *fd);
 
 
 /** \fn int pds_neuron_fscanf(PdsNeuron *Neuron, FILE *fd)
- *  \brief Lee de un archivo de texto los pesos W[i] y el valor de U.
+ *  \brief Lee de un archivo de texto los pesos {W[i]} y  {U, Sigma}.
  *  Ocupando una linea cada uno, y separando los elementos por un TAB.
  *  \param[out] Neuron La neurona a escribir.
  *  \param[in,out] fd Manejador del fichero a leer.
@@ -432,7 +549,7 @@ int pds_neuron_fscanf(PdsNeuron *Neuron, FILE *fd);
 
 
 /** \fn int pds_neuron_fwrite(const PdsNeuron *Neuron, FILE *fd)
- *  \brief Guarda en un archivo binario los pesos W[i] y el valor de U.
+ *  \brief Guarda en un archivo binario los pesos W[i], el valor de U y Sigma.
  *  \param[in] Neuron La neurona a leer.
  *  \param[in,out] fd Manejador del fichero binario a escribir.
  *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL o fd==NULL). 
@@ -442,7 +559,7 @@ int pds_neuron_fwrite(const PdsNeuron *Neuron, FILE *fd);
 
 
 /** \fn int pds_neuron_fread(PdsNeuron *Neuron, FILE *fd)
- *  \brief Lee de un archivo binario los pesos W[i] y el valor de U.
+ *  \brief Lee de un archivo binario los pesos W[i], el valor de U y Sigma.
  *  \param[out] Neuron La neurona a escribir.
  *  \param[in,out] fd Manejador del fichero binario a leer.
  *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL o fd==NULL). 
@@ -450,6 +567,13 @@ int pds_neuron_fwrite(const PdsNeuron *Neuron, FILE *fd);
  */
 int pds_neuron_fread(PdsNeuron *Neuron, FILE *fd);
 
+//@}
+
+/** @name pds_neuron_<free>
+ *  Limpiando memoria.
+ *
+ * @{
+ */
 
 /** \fn void pds_neuron_free(PdsNeuron *Neuron)
  *  \brief Libera una neurona de tipo puntero PdsNeuron.
@@ -468,6 +592,8 @@ void pds_neuron_free(PdsNeuron *Neuron);
  */
 void pds_neuron_destroy(PdsNeuron **Neuron);
 
+//@}
+
 /**
  * @}
  */
